/**
 *	@file	StringEncoder.cpp
 *	@brief	UTF8,SJIS,EUC URLGR[h.
 */

#include "stdafx.h"
#include "StringEncoder.h"



/* -----------------------------------------------------------------------
 *	֐	F	CUnicodeF::sjis_to_utf8
 *	@\TvF	SJIS -> UTF-8 ֕̃R[hϊ
 *		F	(I)  char *pAnsiStr - ϊSJISւ̃|C^
 *				(O)  int *nBytesOut - ϊ㕶̃oCg
 *	߂l	F	char* ϊUTF-8ւ̃|C^
 *
 *	TvF
 * -----------------------------------------------------------------------
 */
LPSTR CURLEncoder::sjis_to_utf8(LPCSTR pAnsiStr, int *nBytesOut)
{
	int 	nNum;
	int 	nBytes;

	wchar_t *pwcWork = sjis_to_utf16be( pAnsiStr, &nNum);
	char *	 pcUtf8  = utf16be_to_utf8( pwcWork, &nBytes);

	//x free( pwcWork);
	operator delete(pwcWork);

	*nBytesOut = nBytes;
	return pcUtf8;
}



/* -----------------------------------------------------------------------
 *	֐	F	CUnicodeF::sjis_to_utf16be
 *	@\TvF	SJIS -> UTF-16 ֕̃R[hϊ
 *		F	(I)    char *pAnsiStr - ϊSJISւ̃|C^
 *				(O)    int *nBytesOut - ϊ㕶̃oCg
 *	߂l	F	wchar_t* ϊUTF-16BEւ̃|C^
 *
 *	TvF
 * -----------------------------------------------------------------------
 */
LPWSTR CURLEncoder::sjis_to_utf16be(LPCSTR pAnsiStr, int *nBytesOut)
{
	size_t		len;
	wchar_t*	pUcsStr = NULL;

	if (!pAnsiStr)
		return NULL;

	setlocale(LC_ALL, "Japanese");	// ꂪȂUnicodeɕϊȂI

	len 	   = strlen( pAnsiStr);
	*nBytesOut = int( sizeof (wchar_t) * (len) );

	pUcsStr    = (wchar_t *) operator new(*nBytesOut + 4);

	if (!pUcsStr)
		return NULL;

	memset(pUcsStr, 0, *nBytesOut + 4);
	mbstowcs(pUcsStr, pAnsiStr, len + 1);

	return pUcsStr;
}



/* -----------------------------------------------------------------------
 *	֐	F	CUnicodeF::utf16be_to_utf8
 *	@\TvF	UTF-16 -> UTF-8 ֕̃R[hϊ
 *		F	(I) wchar_t *pUcsStr - ϊUTF-16BEւ̃|C^
 *				(O) int *nBytesOut - ϊ㕶̃oCg
 *	߂l	F	char* ϊUTF-8ւ̃|C^
 *
 *	TvF
 * -----------------------------------------------------------------------
 */
LPSTR CURLEncoder::utf16be_to_utf8(LPCWSTR pUcsStr, int *nBytesOut)
{
	size_t		nUcsNum;
	char *		pUtf8Str;

	nUcsNum    = wcslen(pUcsStr);

	*nBytesOut = (int) utf16be_to_utf8_sub( NULL, pUcsStr, nUcsNum, TRUE);

	pUtf8Str   = (char *) operator new(*nBytesOut + 6);


	if (pUtf8Str == NULL)
		return NULL;

	memset(pUtf8Str, 0, *nBytesOut + 6);
	utf16be_to_utf8_sub( pUtf8Str, pUcsStr, (int)nUcsNum, FALSE);

	return pUtf8Str;
}



/// Unicode(UTF-16) -> UTF-8 
int CURLEncoder::utf16be_to_utf8_sub(LPSTR pUtf8, LPCWSTR pUcs2, int nUcsNum, BOOL bCountOnly)
{
	int 	nUcs2;
	int 	nUtf8 = 0;

	for (nUcs2 = 0; nUcs2 < nUcsNum; nUcs2++) {
		if ( (unsigned short) pUcs2[nUcs2] <= 0x007f ) {
			if (bCountOnly == FALSE) {
				pUtf8[nUtf8] = (pUcs2[nUcs2] & 0x007f);
			}

			nUtf8 += 1;
		} else if ( (unsigned short) pUcs2[nUcs2] <= 0x07ff ) {
			if (bCountOnly == FALSE) {
				pUtf8[nUtf8]	 = ( (pUcs2[nUcs2] & 0x07C0) >> 6 ) | 0xc0; 	// 2002.08.17 C
				pUtf8[nUtf8 + 1] = (pUcs2[nUcs2] & 0x003f) | 0x80;
			}

			nUtf8 += 2;
		} else {
			if (bCountOnly == FALSE) {
				pUtf8[nUtf8]	 = ( (pUcs2[nUcs2] & 0xf000) >> 12 ) | 0xe0;	// 2002.08.04 C
				pUtf8[nUtf8 + 1] = ( (pUcs2[nUcs2] & 0x0fc0) >> 6 ) | 0x80;
				pUtf8[nUtf8 + 2] = (pUcs2[nUcs2] & 0x003f) | 0x80;
			}

			nUtf8 += 3;
		}
	}

	if (bCountOnly == FALSE) {
		pUtf8[nUtf8] = '\0';
	}

	return nUtf8;
}



void CURLEncoder::URLEncode_EUC(CString &str)
{
	CString 	strRet;
	CString 	strTmp;
	char *		buf = (char *) (LPCTSTR) str;
	int 		ctext;
	int 		hi, lo, ehi, elo;

	while (*buf) {
		ctext = *buf & 0xFF;

		if (ctext == 0x20) {
			//Xy[X"%20"ɕϊ
			strRet += _T("%20");
		} else if ( NotEncodeChar(ctext) ) {
			//GR[hsKvASCII
			strTmp.Format("%c", ctext);
			strRet += strTmp;
		} else if (0x21 <= ctext && ctext <= 0x7E) {
			//GR[hASCII
			strTmp.Format("%%%x", ctext);
			strRet += strTmp;
		} else if (0xA0 <= ctext && ctext <= 0xDF) {
			//pJisjis1oCg,euc2oCgȈ
			strTmp.Format("%%%x%%%x", 0x8E, ctext);
			strRet += strTmp;
		} else {
			hi		= ctext & 0xFF;
			lo		= *++buf & 0xFF;

			if (0x40 <= lo && lo <= 0x9E) {
				if (0x81 <= hi && hi <= 0x9F)
					ehi = (hi << 1) - 0x61;
				else if (0xE0 <= hi && hi <= 0xEF)
					ehi = (hi << 1) - 0xE1;
				else
					ehi = '?';		//error?
				if (lo >= 0x80)
					elo = lo + 0x60;
				else
					elo = lo + 0x61;
			} else if (0x9F <= lo && lo <= 0xFC) {
				if (0x81 <= hi && hi <= 0x9F)
					ehi = (hi << 1) - 0x60;
				else if (0xE0 <= hi && hi <= 0xEF)
					ehi = (hi << 1) - 0xE0;
				else
					ehi = '?';

				elo = lo + 0x2;
			} else {
				ehi = '?';
				elo = '?';
			}

			strTmp.Format("%%%x%%%x", ehi & 0xFF, elo & 0xFF);
			strRet += strTmp;
		}

		buf++;
	}

	str = strRet;
}



/// GR[hKv(0-9,a-z,A-Z,-_@.*)`FbN
BOOL CURLEncoder::NotEncodeChar(int code)
{
	if (  (0x30 <= code && code <= 0x39)									//0-9
	   || (0x40 <= code && code <= 0x5A)									//@A-Z (0x39̎0x3A炭Ⴞ)
	   || (0x61 <= code && code <= 0x7A)									//a-z
	   || code == 0x2A || code == 0x2D || code == 0x2E || code == 0x5F )	//*-._
	{
		return TRUE;
	}

	return FALSE;
}



/// U.H(DonutP 3.5ւ̒ǉ)̃R[h searchbar.h/EncodeStringړ)
void CURLEncoder::URLEncode_SJIS(CString &str)
{
	CString strRet;
	int 	nStrLen = str.GetLength();

	for (int ii = 0; ii < nStrLen; ii++) {
		CString strOne;
		WCHAR	cText = str[ii];
		cText = cText & ~0xff00;			//?? cText &= 0xFF ??;

		if (cText == 0x20) {
			strRet += _T("%20");
		} else if ( (cText >= 0x40 && cText <= 0x5A) || (cText >= 0x61 && cText <= 0x7A)
				  || (cText >= 0x30 && cText <= 0x39) || cText == 0x2A || cText == 0x2D || cText == 0x2E || cText == 0x5F )
		{
			strOne.Format("%c", cText);
			strRet += strOne;
		} else {
			strOne.Format("%%%x", cText);
			strRet += strOne;
		}
	}

	str = strRet;
}



void CURLEncoder::URLEncode_UTF8(CString &str)
{
	CString 		strRet;
	CString 		strTmp;
	int 			BufSize = 0;
	unsigned short	cText;
	char *			pBuf;

	pBuf = sjis_to_utf8( (LPCTSTR) str, &BufSize );

	for (int i = 0; i < BufSize; i++) {
		cText = *(unsigned short *) pBuf;

		if (cText == 0x20) {
			strRet += _T("%20");
		} else if ( NotEncodeChar(cText) ) {
			strTmp.Format("%c", cText);
			strRet += strTmp;
		} else {
			strTmp.Format("%%%2x", *(pBuf + i) & 0xFF);
			strRet += strTmp;
		}
	}

	operator delete(pBuf);

	str  = strRet;
}

