#ifndef __MTLPROFILE_H__
#define __MTLPROFILE_H__

// MTL Version 0.03
// Copyright (C) 2000 MB<mb2@geocities.co.jp>
// All rights unreserved.
//
// This file is a part of Mb Template Library.
// The code and information is *NOT* provided "as-is" without
// warranty of any kind, either expressed or implied.
//
// Last updated: August 16, 2000

//Donut̃\[XR[hCIniSectionNXMFCœ悤ɂ̂ł
//by minit

#pragma once



////////////////////////////////////////////////////////////////////////////
// CIniSection

class CIniSection
{
public:
// Constructor/destructor
	CIniSection();
	~CIniSection();
	
// Attributes
public:
	CString m_strFileName, m_strSectionName;
	
// Operations
public:
	// for "Profile" concepts 
	LONG SetValue(DWORD dwValue, LPCTSTR lpszValueName);
	LONG QueryValue(DWORD& dwValue, LPCTSTR lpszValueName);
	LONG QueryValue(LPTSTR szValue, LPCTSTR lpszValueName, DWORD* pdwCount);
	LONG SetValue(LPCTSTR lpszValue, LPCTSTR lpszValueName);
	LONG DeleteValue(LPCTSTR lpszValueName);

	// original
	void Open(const CString& strFileName, const CString& strSectionName);
	void Close();

// Implementation
protected:
	enum { _nDefault = 0xdeadbeef };// magic number

	BOOL _IsOpen()
	{
		if (!m_strFileName.IsEmpty() && !m_strSectionName.IsEmpty())
			return TRUE;
		else
			return FALSE;
	}
};

inline CIniSection::CIniSection()
{
}

inline CIniSection::~CIniSection()
{
}

inline void CIniSection::Close()
{
	m_strFileName.Empty();
	m_strSectionName.Empty();
}

inline void CIniSection::Open(const CString& strFileName, const CString& strSectionName)
{
	ASSERT(!strFileName.IsEmpty());
	ASSERT(!strSectionName.IsEmpty());
	ASSERT(!_IsOpen());

	m_strFileName = strFileName;
	m_strSectionName = strSectionName;
}

inline LONG CIniSection::QueryValue(DWORD& dwValue, LPCTSTR lpszValueName)
{
	ASSERT(_IsOpen());
	
	UINT nValue = ::GetPrivateProfileInt(m_strSectionName, lpszValueName, _nDefault, m_strFileName);
	if (nValue == _nDefault)
		return ERROR_CANTREAD;

	dwValue = nValue;
	return ERROR_SUCCESS;
}

inline LONG CIniSection::QueryValue(LPTSTR szValue, LPCTSTR lpszValueName, DWORD* pdwCount)
{
	ASSERT(pdwCount != NULL);
	ASSERT(_IsOpen());
	
	ASSERT(szValue != NULL);// Ini section can't determine dwCount.

	DWORD dw = ::GetPrivateProfileString(m_strSectionName, lpszValueName,
		_T(""), szValue, *pdwCount, m_strFileName);

	if (dw == 0)
		return ERROR_CANTREAD;

	return ERROR_SUCCESS;
}

inline LONG CIniSection::SetValue(DWORD dwValue, LPCTSTR lpszValueName)
{
	ASSERT(lpszValueName != NULL);// don't make section deleted
	ASSERT(_IsOpen());
	
	TCHAR szT[16];
	::wsprintf(szT, _T("%d"), dwValue);
	if (::WritePrivateProfileString(m_strSectionName, lpszValueName, szT, m_strFileName))
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}

inline LONG CIniSection::SetValue(LPCTSTR lpszValue, LPCTSTR lpszValueName)
{
	ASSERT(lpszValue != NULL);
	ASSERT(_IsOpen());
	
	ASSERT(m_strFileName.GetLength() < 4095); // can't read in bigger
	if (::WritePrivateProfileString(m_strSectionName, lpszValueName, lpszValue, m_strFileName))
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}

inline LONG CIniSection::DeleteValue(LPCTSTR lpszValueName)
{
	if (::WritePrivateProfileString(m_strSectionName, lpszValueName, NULL, m_strFileName))
		return ERROR_SUCCESS;
	else
		return ERROR_CANTWRITE;
}

/*
inline bool MtlIniDeleteSection(const CString& strIniFileName, const CString& strSectionName)
{
	return ::WritePrivateProfileString(strSectionName, NULL, NULL, strIniFileName) ? true : false;
}

inline bool MtlIniDeleteAllSection(const CString& strIniFileName, const CString& strSectionName)
{
	return true;
}
*/

#endif //__MTLPROFILE_H__